option show_stats;
option solver gurobi_ampl;

## Model parameters
param p; # Number of processors
param b; # Child per node
param L = log(p) / log(b);
param alpha_static;

## Optimization
param M; # Crown maximal makespan
param zeta; # Weight for static energy consumption
param epsilon; # Energy saving at idle time

set P = 1..p; # Set of processors
set B = 1..b; # Set of child at each node except leaves

## Convenient variables
#param pob{i in P} = if log(i) / log(b) = floor(log(i) / log(b)) then 1 else 0; # Powers Of B
param q = 2 * p - 1;

## Hierarchical structure of processor groups
set Pi{i in 1..q} within P = if floor(log(i) / log(b)) + 1 <= L then union{j in B} Pi[b * i + j - 1] else {i - b^L + 1};
set G{i in P} = union{j in 1..q} if i in Pi[j] then {j} else {};
param pi{i in 1..q} = card(Pi[i]);
# Matrix equivalent to pi (M + pi = Mpi)
param Mpi{i in 1..q, j in P} = if pi[i] = j then 1 else 0;

## Tasks
param n; # Number of tasks
set T = 1..n; # Task set
param Wi{i in T}; # Task width
param Tau{i in T}; # Task work
param e{i in T, j in P}; # Task efficiency distribution
param c{i in T, j in T}; # Task channels
#param e_max{i in T, j in P} = if Wi[i] >= j then e[i, j] else min(e[i, j], (e[i, j] * Wi[i]) / j); # Maximal task efficiency after a task's given efficiency and width

## Resource allocation
#param wi{i in T} = max(1, min(Wi[i], p)); # Always give maximal resources a task can handle
#param wi{i in T}; # Get resource allocation from allocation.dat
## Matrix equivalent to wi (M + wi = Mwi)
#param Mwi{i in T, j in P} binary = if wi[i] = j then 1 else 0;

## FFrequency
set F;
set FF = union{i in F} {i / min{j in F}j};
param alpha; # FFrequency cost parameter
param fa{i in FF} = i^(alpha) + zeta * i; # Cost of a frequency
param fa_static{i in FF} = i^(alpha_static) + zeta * i; # Cost of a frequency in static energy
 
## FFrequency mapping to tasks
#var f{i in T, j in FF} binary;
#subject to one_frequency_per_task {i in T}: sum{j in FF} f[i, j] = 1; # A task receives exactly one running frequency

## Integrated mapping
var x{i in 1..q, j in T, k in FF} binary; # 3D Matrix for frequency/group mapping
subject to no_mapping_to_bigger_groups{j in T}: sum{i in 1..(p / (b^floor(log(min(Wi[j], p)) / log(2))) - 1)} sum{k in FF} x[i, j, k] = 0; # Every task gets exactly one group and one frequency
subject to one_setting_per_task{j in T}: sum{i in p / (b^floor(log(min(Wi[j], p)) / log(2)))..q} sum{k in FF} x[i, j, k] = 1; # Every task gets exactly one group and one frequency
#subject to no_mapping_to_bigger_groups{j in T}: sum{i in 1..(floor(p / Wi[j]) - 1)} sum{k in FF} x[i, j, k] = 0; # Every task gets exactly one group and one frequency
#subject to one_setting_per_task{j in T}: sum{i in floor(p / Wi[j])..q} sum{k in FF} x[i, j, k] = 1; # Every task gets exactly one group and one frequency
#subject to one_setting_per_task{j in T}: sum{i in 1..q} sum{k in FF} x[i, j, k] = 1; # Every task gets exactly one group and one frequency

## More variables to ease reading of x
var grp{j in T} = sum{i in 1..q} sum{k in FF} x[i, j, k] * i;
var freq{j in T} = sum{i in 1..q} sum{k in FF} x[i, j, k] * k * min{l in F}l;
var _int_frequency{i in T} = freq[i]; # Alias for output with non-confusing name
var frequency{i in T} = round(_int_frequency[i]); # make sure output is integer digits
var _int_group{i in T} = grp[i]; # Alias for output with non-confusing name
var group{i in T} = round(_int_group[i]); # make sure output is integer digits

## Time
var time{ii in P} = sum{i in G[ii]} sum{l in P} Mpi[i, l] * sum{k in FF} sum{j in T} (x[i, j, k] * Tau[j] / (e[j, l] * l * k));
subject to throughput{i in P}: time[i] <= M;

var int_time{i in P} = time[i]; # Alias for output with non-confusing name

## Energy: static, dynamic and weighted global
var int_busy = sum{i in 1..q} sum{l in P} Mpi[i, l] * sum{k in FF} fa[k] * sum{j in T} x[i, j, k] * Tau[j] / (e[j, l] * k);
var int_idle = sum{ii in P} (M - time[ii]) * min{i in FF} fa_static[i];
var int_energy = int_busy + epsilon * int_idle;

## Give allocation output
var wi{i in T} = p / (b ^ floor((log(round(grp[i])) / log(b))));

## Distribute tasks so we reduce the makespan of a schedule
## Actually distribute the work each core is given, where frequency has no incidence; this prevent teh solver from trying to optimize makespan by running tasks faster
## Also prevent the solver from trying to reduce the makespan by parallelizing tasks because it can make it to sacrify energy for better makespan. Assumes parallelized tasks run several times sequentially on all processors and try to balance the resulting load
## Make sure the efficency loss due parallelization choice doesn't create different runtime ratio between tasks when they are parallelized and not parallelized. Penalize a sequential task's load with the efficiency of the chosen parallelization makes sure this ratio is preserved even if all tasks have a different efficiency function, but assuming this function is decreasing from 1 to +infinite.
## In other words, gets rid of values (parallelization time + frequency scaling) that can tempt the solver to modify its solution to achieve a better makespan, but make sure the parallelized work order relationship between tasks is preserved with the work loads we are trying to balance.
var core_work_eq{ii in P} = sum{i in G[ii]} sum{l in P} Mpi[i, l] * sum{k in FF} sum{j in T} (x[i, j, k] * Tau[j] / e[j, l]); ## core work equivalent (equivalent = different value than work, but having the same ratio as between each tasks and cores' work after parallelization)
var max_core_work_eq;
subject to balance_core_work_eq{i in P}: core_work_eq[i] <= max_core_work_eq;

## Wrap energy optimization and task balance together

#var energy_balance = max_core_work_eq + int_energy; ## Try to balance tasks
var energy_balance = int_energy; ## Only take energy efficiency into account

minimize target: energy_balance;
