/*
 Copyright 2015 Nicolas Melot

 This file is part of Static-Dynamic-Busy.

 Static-Dynamic-Busy is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Static-Dynamic-Busy is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Static-Dynamic-Busy. If not, see <http://www.gnu.org/licenses/>.

*/


#include <iostream>
#include <cstdlib>
#include <fstream>
#include <string>
#include <boost/regex.hpp>
#include <iomanip>

#include <pelib/Platform.hpp>
#include <pelib/GraphML.hpp>
#include <pelib/XMLSchedule.hpp>

#include <pelib/AmplInput.hpp>
#include <pelib/AmplOutput.hpp>

#include <pelib/AmplOutputScalar.hpp>
#include <pelib/AmplOutputVector.hpp>
#include <pelib/AmplOutputSet.hpp>
#include <pelib/AmplOutputMatrix.hpp>

#include <pelib/AmplInputScalar.hpp>
#include <pelib/AmplInputVector.hpp>
#include <pelib/AmplInputSet.hpp>
#include <pelib/AmplInputMatrix.hpp>

#ifdef debug
#undef debug
#endif

#if 1
#define debug(var) cout << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #var << " = \"" << var << "\"" << endl;
#else
#define debug(var)
#endif

using namespace pelib;
using namespace std;

typedef struct
{
	double alpha, epsilon, kappa, eta;
	char *taskgraph, *platform, *schedule;
	bool describe, switch_off;
} args_t;

static args_t
parse_arguments(char **argv)
{
	args_t args;

	// default values
	args.alpha = 3.0;
	args.epsilon = 0.649;
	args.kappa = 52.639161335;
	args.eta = 0.018611;
	args.describe = false;
	args.switch_off = true;

	for(argv++; argv[0] != NULL; argv++)
	{
		if(string(argv[0]).compare("--alpha") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.alpha;
			continue;
		}
		if(string(argv[0]).compare("--epsilon") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.epsilon;
			continue;
		}
		if(string(argv[0]).compare("--kappa") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.kappa;
			continue;
		}
		if(string(argv[0]).compare("--eta") == 0)
		{
			argv++;
			stringstream ss(argv[0]);
			ss >> args.eta;
			continue;
		}
		if(string(argv[0]).compare("--taskgraph") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.taskgraph = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--platform") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.platform = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--schedule") == 0)
		{
			argv++;
			if(argv[0][0] == '-') { argv--;	continue; }
			args.schedule = argv[0];
			continue;
		}
		if(string(argv[0]).compare("--label") == 0)
		{
			args.describe = 1;
			continue;
		}
		if(string(argv[0]).compare("--no-switch-off") == 0)
		{
			args.switch_off = false;
			continue;
		}
	}

	return args;
}

int
main(int argc, char **argv)
{
	args_t args = parse_arguments(argv);

	if(args.describe)
	{
		// Just output a description string
		cout << "Stat./dyn.a=" << args.alpha << ",ep=" << args.epsilon << ",k=" << args.kappa << ",eta=" << args.eta << "(act.)" << endl;
	}
	else
	{
		AmplOutput output(AmplOutput::floatHandlers());

		cout << setprecision(6)
		<< setiosflags(ios::fixed)
		<< setiosflags(ios::showpoint);

		std::ifstream tgStream;
		tgStream.open (args.taskgraph, std::ios::in);
		std::ifstream archStream;
		archStream.open (args.platform, std::ios::in);
		std::ifstream schedStream;
		schedStream.open (args.schedule, std::ios::in);
		Algebra rec = AmplInput(AmplInput::floatHandlers()).parse(archStream);
		Platform arch(rec);
		Taskgraph *tg = GraphML().parse(tgStream);
		rec = rec.merge(tg->buildAlgebra(arch));
		Schedule *sched = XMLSchedule().parse(schedStream);
		rec = rec.merge(sched->buildAlgebra());
		tgStream.close();
		archStream.close();
		schedStream.close();

		int n = rec.find<Scalar<float> >("n")->getValue();
		int p = rec.find<Scalar<float> >("p")->getValue();
		float M = rec.find<Scalar<float> >("M")->getValue();
		Vector<int, float> tau = rec.find<Vector<int, float> >("Tau");
		Vector<int, float> Wi = rec.find<Vector<int, float> >("Wi");
		Vector<int, float> frequency = rec.find<Vector<int, float> >("frequency");
		Set<float> F = rec.find<Set<float> >("F");
		Matrix<int, int, float> e = rec.find<Matrix<int, int, float> >("e");
		Matrix<int, int, float> schedule = rec.find<Matrix<int, int, float> >("schedule");
		vector<int> width(n, 0);
		vector<double> busy_task(n, 0);
		vector<double> area_task(n, 0);
		vector<double> met_task(n, 0);
		
		float lowest_freq = *(F.getValues().begin());
		args.kappa = args.kappa * lowest_freq;
		args.eta = args.eta * (args.epsilon * pow(lowest_freq, args.alpha) + (1 - args.epsilon) * (lowest_freq + args.kappa)) / args.epsilon;
		double idle_energy = p * M * pow(lowest_freq, args.alpha);
		double busy_energy = 0;

		map<float, int> core_active;
		int total_core_active = 0;

		// First get the width of each task
		for(map<int, map<int, float> >::const_iterator i = schedule.getValues().begin(); i != schedule.getValues().end(); i++)
		{
			//int core = i->first;
			for(map<int, float>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
			{
				int task = j->second;
				//cout << "Core " << core << ", task " << task << endl;
				if(task > 0) width[task - 1] = width[task - 1] + 1;
			}

			// Initialize the use of core to 0
			core_active[i->first] = 0;
		}

		// Now evaluate the quality of this schedule
		float total_area = 0;
		for(map<int, map<int, float> >::const_iterator i = schedule.getValues().begin(); i != schedule.getValues().end(); i++)
		{
			//int core = i->first;
			for(map<int, float>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
			{
				int index = j->first;
				int task = j->second;
				//cout << "Core " << core << ", task " << task << endl;
				if(task > 0)
				{
					//float tau_t = tau.find(task);
					float width_t = width[task - 1];
					//double e_wt = e.find(width_t, task);
					//float f_t = frequency.find(task);
					double time = (double)tau.find(task) / (width[task - 1] * e.find(width_t, task) * (double)frequency.find(task));
					total_area += time;
					//busy_energy += time * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa));
					//busy_energy += time * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa) + (1 - args.epsilon) * args.eta);
					busy_energy += time * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa)) / args.epsilon;
					//busy_task[task - 1] += time * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa)) / args.epsilon;
					//area_task[task - 1] += time;
					//met_task[task - 1]++;

					//if(met_task[task -1] == width_t)
					{
						//cout << "Task " << task << ", tau " << tau.find(task) << ", width " << width_t << ", e " << e.find(width_t, task) << ", time " << time << ", f " << frequency.find(task) << ", energy " << area_task[task - 1] * (args.epsilon * pow(frequency.find(task), args.alpha) + (1 - args.epsilon) * (frequency.find(task) + args.kappa)) / args.epsilon << endl;
					}
					// cout << static_energy << endl;
					// cout << dynamic_energy << endl;

					// Mark this core as active
					if(core_active[i->first] == 0)
					{
						core_active[i->first] = 1;
						total_core_active++;
					}
				}
			}
		}
		//idle_energy = (M * (total_core_active) - total_area) * args.eta;
		if(args.switch_off)
		{
			idle_energy = (M * (total_core_active) - total_area) * args.eta;
		}
		else
		{
			idle_energy = (M * p - total_area) * args.eta;
		}

		typedef std::map<int, float> RowType;
		typedef std::map<int, RowType> MatrixType;

		RowType xs, ys;
		xs[0] = 0;
		xs[1] = 1;
		ys[0] = args.epsilon * busy_energy + (1 - args.epsilon) * idle_energy;
		ys[1] = ys[0];

		MatrixType matrix;
		matrix[0] = xs;
		matrix[1] = ys;

		AmplOutput out(AmplOutput::floatHandlers());
		Matrix<int, int, float> energy("time__power", matrix);
		Matrix<int, int, float> energy_transposed = energy.transpose();
		out.dump(cout, &energy_transposed);

		//cout << std::setprecision(10);
		//cout << dynamic_energy + args.epsilon * static_energy << " " << rec.find<Scalar<float> >("time")->getValue() << endl;
		//cout << "total_active_core = " << total_core_active << endl;
		//cout << "idle_energy = " << idle_energy << endl;
		//cout << "busy_energy = " << busy_energy << endl;
		//cout << "alpha = " << args.alpha << endl;
		//cout << "epsilon = " << args.epsilon << endl;
		//cout << "kappa = " << args.kappa << endl;
		//cout << "eta = " << args.eta << endl;
		//cout << "M = " << M << endl;
		//cout << ( args.switch_off ? "Switch off unused cores" : "Leave all cores active" ) << endl;
		//cout << "busy_task = ";
		//for(size_t i = 0; i < n; i++)
		//{
		//	cout << i + 1 << " " << busy_task[i] << "	";
		//}
		//cout << endl;
	}

	return EXIT_SUCCESS;
}
